# Generated by Django 3.2 on 2021-04-23 19:06

from django.db import migrations, models


class Migration(migrations.Migration):
    dependencies = [
        ('eav', '0003_auto_20210404_2209'),
    ]

    operations = [
        migrations.AlterField(
            model_name='value',
            name='value_bool',
            field=models.BooleanField(blank=True, null=True),
        ),
    ]
